﻿using System;
using System.Drawing;
using OpenTK;
using OpenTK.Graphics.OpenGL;
using Utils;
using OpenTK.Input;
using System.Collections.Generic;

namespace DrawPointsByClick
{
    class MainWindow : GameWindow
    {
        private int program;
        private bool canDraw = false;
        private int a_Position;

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Console.WriteLine("Vendor: " + GL.GetString(StringName.Vendor));
            Console.WriteLine("OpenGL version: " + GL.GetString(StringName.Version));

            Title = "Draw Points By Click";
            Width = 400;
            Height = 400;

            // Load shaders from files
            string vShaderSource = null;
            string fShaderSource = null;
            ShaderLoader.LoadShader("./Shaders/VertexShader.glsl", out vShaderSource);
            ShaderLoader.LoadShader("./Shaders/FragmentShader.glsl", out fShaderSource);
            if (vShaderSource == null || fShaderSource == null)
            {
                Logger.Append("Failed to load shaders from files");
                return;
            }

            // Initialize the shaders
            if (!ShaderLoader.InitShaders(vShaderSource, fShaderSource, out program))
            {
                Logger.Append("Failed to initialize the shaders");
                return;
            }

            // Get the storage location of a_Position
            a_Position = GL.GetAttribLocation(program, "a_Position");
            if (a_Position < 0)
            {
                Logger.Append("Failed to get the storage location of a_Position");
                return;
            }

            // Pass coordinates of point to a_Position
            GL.VertexAttrib2(a_Position, 0f, 0.5f);

            // Enable Point Size
            GL.Enable(EnableCap.ProgramPointSize);

            // Specify the color for clearing the canvas
            GL.ClearColor(Color.FromArgb(255, 131, 182, 199));

            canDraw = true;
        }

        protected override void OnRenderFrame(FrameEventArgs e)
        {
            GL.Viewport(0, 0, Width, Height);
            base.OnRenderFrame(e);

            // Clear the canvas with current color
            GL.Clear(ClearBufferMask.ColorBufferBit);

            if (canDraw)
            {
                foreach (var p in points)
                {
                    // Pass coordinates of the point to a_Position
                    GL.VertexAttrib2(a_Position, p.X, p.Y);

                    // Draw the point
                    GL.DrawArrays(PrimitiveType.Points, 0, 1);
                }
            }

            GL.Flush();
            SwapBuffers();
        }

        private List<Vector2> points = new List<Vector2>();
        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            base.OnMouseDown(e);

            if (e.Button == MouseButton.Left)
            {
                float x = (e.X - Width / 2f) / (Width / 2f);
                float y = -(e.Y - Height / 2f) / (Height / 2f);

                // Store the coordinates to point list
                points.Add(new Vector2(x, y));

                //Console.WriteLine(x + " " + y);
            }
        }
    }
}
